function DisplayImageElement(ContentElement){
    var self = this; self.name = "DisplayImageElement"; // MANDATORY

    this.reference = -1;
    this.type = -1;
    this.switch = 0;
    this.relative_path = false;
    this.force_http_source = false;
    this.die = true;
    this.time_to_die = 1500;

    this.loaded = false;

    this.x = 0;
    this.y = 0;
    this.w = "100%";
    this.h = "100%";
    this.z = 1;

    this.ui = false;
    this.uiTimer = null;
    this.uiStartTime = 0;

    this.source = ContentElement.getSource();
    this.duration = ContentElement.getDuration();
    this.relative_path = ContentElement.relative_path;
    this.path = ContentElement.path;
    this.tag = "";
    this.timer = null;
    this.start_timer = 0;
    this.start_current_time = 0;

    this.update = function(ContentElement){
        self.content = ContentElement;
        self.source = ContentElement.getSource();
        self.duration = ContentElement.getDuration();
        self.relative_path = ContentElement.relative_path;
        self.path = ContentElement.path;
    };
    
    this.createImage = function(layer){
      try{
        $("#"+self.tag).hide();
        $("#"+self.tag).remove();
      }catch(e){
        self.log("TAG does not exist");
      }
      $(".stage").append('<img class="player-media" id="'+self.tag+'" src="">');
    };

    this.load = function(){
        self.loaded = false;
        self.createImage();
        self.log("Load on tag : "+$("#"+self.tag).attr("id"));
        $("#"+self.tag).off("error",self.onElementLoadError);
        $("#"+self.tag).off("load",self.onElementLoadComplete);
        $("#"+self.tag).off();
        $("#"+self.tag).attr("src",""); // IMPORTANT !!!! FOR SOC !!
        $("#"+self.tag).attr("src","javascript:void(0)"); // IMPORTANT !!!! FOR SOC !!
        $("#"+self.tag).one("load",self.onElementLoadComplete);
        $("#"+self.tag).one("error",self.onElementLoadError);
        var source = self.path;
        var ext = source.substring(source.lastIndexOf(".")+1,source.length);
        switch(ext){
            case "mjpeg":
                source = self.OS.readFile(source,self.onFileRead);
            break;
            default:
                self.log("Loading : "+source+" on tag :"+self.tag);
                $("#"+self.tag).attr("src",source);
            break;

        }
    };

    this.onFileRead = function(result){
        self.log("Loading : "+result);
        $("#"+self.tag).attr("src",result);
    };


    this.onElementLoadComplete = function(event){
        $("#"+self.tag).off("error",self.onElementLoadError);
        $("#"+self.tag).off("load",self.onElementLoadComplete);
        $("#"+self.tag).off();
        self.playstatus = "STOP";
        self.log("Loaded on tag :"+self.tag);
        self.loaded = true;
        $(self).trigger("DisplayElementLoaded");
    };

    this.onElementLoadError = function(event){
        $("#"+self.tag).off("error",self.onElementLoadError);
        $("#"+self.tag).off();
        self.notifyError(self.ERROR['DISPLAY_IMAGE_ELEMENT']+1,self.name+": Function onElementLoadError() failed : could not load :"+self.path);
        self.disable();
        self.playstatus = "INVALID";
        self.log("Error loading on tag : "+$(event.currentTarget).attr("id"));
        $(self).trigger("DisplayElementError");
    };

    this.disable = function(){
        $("#"+self.tag).off("error");
        $("#"+self.tag).off("load");
        $("#"+self.tag).off();
    };

    this.play = function(){
        $("#"+self.tag).off("error");
        $("#"+self.tag).off("load");
        $("#"+self.tag).off();
        self.log("Play : "+self.reference+" with duration : "+self.duration+" on tag :"+self.tag);
        if(self.duration > 0){
          if(self.start_current_time > 0){
            self.duration = self.duration*1000 - self.start_current_time;
            self.duration = parseInt(self.duration/1000);
            self.start_current_time = 0;
            console.log("Start Set new timeout : "+self.duration);
          }
          self.timer = setTimeout(self.timeout,self.duration*1000);
        }else{
          self.timer = setTimeout(self.timeout,10000);
        }
        self.start_timer = new Date().getTime();
        $("#"+self.tag).show();
        $("#"+self.tag).css("visibility","visible");
        $(self).trigger("DisplayElementPlay");
        self.uiStartTime = Date.now();
    };

    this.timeout = function(){
        self.log("Finish on tag :"+self.tag);
        clearTimeout(self.timer);
        self.start_timer = 0;
        if(self.die){
            self.playstatus = "STOP";
            $(self).trigger("DisplayElementFinish");
        }
    };

    this.connect = function(ui){
        /*
        self.ui = null;
        self.ui = ui;
        clearInterval(self.uiTimer);
        if(self.ui){
            self.ui.unloadTime();
            self.uiTimer = setInterval(self.updateTime,200);
        }
        */
    };

    this.disconnect = function(){
        clearInterval(self.uiTimer);
        if(self.ui){
            self.ui.unloadTime();
        }
        self.ui = null;
    };

    this.updateTime = function(){
        var current = (Date.now() - self.uiStartTime) / 1000;
        self.ui.loadTime(current,self.duration);
    };

    this.clear_source = function(){
        $("#"+self.tag).attr("src","");
        try{
          $("#"+self.tag).remove();
        }catch(e){
          self.log("TAG does not exist");
        }
    };

    // COMMON
    this.vanish = function(){
        clearInterval(self.uiTimer);
        $("#"+self.tag).hide();
        $("#"+self.tag).css("z-index","0");
        $("#"+self.tag).css("visibility","hidden");
        self.log("hide on tag : "+self.tag);
        //self.clear_source();
    };

    this.destroy = function(time){
        self.loaded = false;
        clearTimeout(self.timer);
        self.disconnect();
        if(typeof(time) === 'undefined'){ time = 0; }
        if(time == 0){
            self.vanish();
            self.clear_source();
        }else{
            setTimeout(function(){
                self.vanish();
                self.clear_source();
            },time);
        }
        self.disable();
        self.log(self.reference+" destroyed ... on tag : "+self.tag);
    };

    this.modifyViewer = function(x,y,w,h,z){
        if(typeof(z) === 'undefined'){z=1;}
        self.x = x;
        self.y = y;
        self.w = w;
        self.h = h;
        self.z = z;
    };

    this.setViewer = function(){
        $("#"+self.tag).css("left",self.x);
        $("#"+self.tag).css("top",self.y);
        $("#"+self.tag).css("width",self.w);
        $("#"+self.tag).css("height",self.h);
        $("#"+self.tag).css("z-index",self.z);
    };
    
    this.getCurrentTime = function(){
      return new Date().getTime() - self.start_timer;
    };
}
DisplayImageElement.prototype = new SystemController();
